# -*- coding: utf-8 -*-

from .taskresult import TaskResult
from .fields import MsgFields as MSGF, MsgErrs as MERR
from .exceptions import TaskResultFailException


class TaskResultSet(dict):
    """
    JobResult is a dict for storing TaskResults with convenience functions.
    """
    def __init__(self, target_id, job_id, data=None):
        self.setdefault(MSGF.TRS_META, {})

        # target_id and job_id are stored internally as strs, since this
        # is how they get json encoded. To API user, they're converted to
        # ints when using .target_id, .job_id, get_task_tmoid() etc.
        try:
            target_id = str(int(target_id))
        except TypeError:
            raise ValueError("Target id must be a number")

        try:
            job_id = str(int(job_id))
        except TypeError:
            raise ValueError("Job id must be a number")

        self[MSGF.TRS_META][MSGF.TRS_META_TARGET_ID] = target_id
        self[MSGF.TRS_META][MSGF.TRS_META_JOB_ID] = job_id

        dict.__init__(self)

        if data and not isinstance(data, dict):
            raise ValueError("data must be a dictionary to import")
        elif data:
            self.update(data)

    @classmethod
    def init_from_dict(cls, data):
        """
        Init a TaskResult set from a a (likely JSON deserialized) dict.
        """
        target_id = data[MSGF.TRS_META][MSGF.TRS_META_TARGET_ID]
        job_id = data[MSGF.TRS_META][MSGF.TRS_META_JOB_ID]

        return cls(target_id, job_id, data=data)

    @property
    def target_id(self):
        """
        Return an int representing target_id of Job.
        """
        try:
            return int(self[MSGF.TRS_META][MSGF.TRS_META_TARGET_ID])
        except KeyError:
            raise ValueError("Target ID not set!")
        except TypeError:
            raise ValueError("Target ID muse be an int")

    @property
    def job_id(self):
        """
        Return an int representing job_id of Job.
        """
        try:
            return int(self[MSGF.TRS_META][MSGF.TRS_META_JOB_ID])
        except KeyError:
            raise ValueError("Job ID not set!")
        except TypeError:
            raise ValueError("Job ID must be an int")

    def add(self, task_result):
        """
        Add a TaskResult to this TaskResultSet.
        """
        missed_field_err = ("TaskResult must specify a {0}. Did you remember "
                            "to finish() the task?")

        if not task_result.output_name:
            raise ValueError(missed_field_err.format("output_name"))
        if not task_result.task_name:
            raise ValueError(missed_field_err.format("task_name"))
        if not task_result.task_version:
            raise ValueError(missed_field_err.format("task_version"))
        if not task_result.tmoid:
            raise ValueError(missed_field_err.format(
                "task monitoring object id"))

        self.setdefault(task_result.output_name, {
            MSGF.TASKRESULTS: {},
            MSGF.META: {}}
        )
        self[task_result.output_name][MSGF.TASKRESULTS][task_result.tmoid] = \
            dict(task_result)

    def add_output_err(self, output_name, err):
        """
        Add an error indicating that there was a TaskResult-level error
        preventing Task from being run (e.g. insufficient inputs)
        Use TaskResult.err when this problem relates to a particular tmoid,
        and TaskResultSet.add_task_err() when none of the tmoids could be
        tested.
        """
        self.setdefault(output_name, {MSGF.TASKRESULTS: {}, MSGF.META: {}})

        self[output_name][MSGF.META].setdefault(MSGF.META_ERR, [])

        self[output_name][MSGF.META][MSGF.META_ERR].append(err)

    def output_success(self, output_name):
        """
        Return a bool indicating if a TaskResult ran without high-level
        errors i.e. testing was attempted for tmoids.
        """
        return self.output_errs(output_name) == []

    def output_errs(self, output_name):
        """
        Return list of errors returned by an output's task.
        """
        try:
            self[output_name][MSGF.META]
        except:
            raise ValueError("%s output not found in message" % output_name)

        return self[output_name][MSGF.META].get(MSGF.META_ERR, [])

    def outputs(self):
        """
        Return a list of outputs contained in a message.
        """
        return filter(lambda k: k != MSGF.TRS_META, self.iterkeys())

    def output_tmoids(self, output_name):
        """
        Return a list of tmoid ints related to a Task result name.
        """
        try:
            self[output_name][MSGF.TASKRESULTS]
        except KeyError:
            raise ValueError("%s task not found in message" % output_name)

        # keys here are json encoded as strs, recast to int
        return [int(k) for k in self[output_name][MSGF.TASKRESULTS].keys()]

    def get_output_tmoid(self, output_name, tmoid, schema=None):
        """
        Get a single TaskResult based on a task name and target monitoring
        ID. Raises exceptions if output_name or tmoid not found.
        Use when you want to get the result back, even if it's failed.
        """
        try:
            tmoid = str(int(tmoid))
        except TypeError:
            raise ValueError("Tmoid must be an int")

        try:
            self[output_name][MSGF.TASKRESULTS][tmoid]
        except KeyError:
            raise ValueError("Could not find output {0} with tmoid {1} in "
                             "results".format(output_name, tmoid))

        tr = TaskResult.init_from_dict(
            output_name,
            tmoid,
            self[output_name][MSGF.TASKRESULTS][tmoid],
            schema=schema
        )

        return tr

    def get_output_tmoid_raise_fail(self, output_name, tmoid, schema=None):
        """
        Returns a TaskResult and error string tuple given output_name and
        tmoid.
        Returns None for TaskResult if there was any problem with the
        TaskResult.
        Use when you want a hard raise when a task result failed.
        """
        try:
            tr = self.get_output_tmoid(output_name, tmoid, schema=schema)
            if not tr.success:
                raise TaskResultFailException(
                    MERR.RESULT_FAIL.format(tr.err))
            return tr
        except Exception:
            raise

    def get_output_tmoid_or_none(self, output_name, tmoid, schema=None):
        """
        Returns a TaskResult and error string tuple given output_name and
        tmoid.
        Returns None for TaskResult if there was any problem with the
        TaskResult.
        Use when you want a soft failure when a task failed.
        """
        tr = None
        err = None

        try:
            tr = self.get_output_tmoid_raise_fail(
                output_name, tmoid, schema=schema)
        except Exception as e:
            err = e
        return ((tr, err))



from .fields import (MsgFields as _MsgFields, MsgErrs as _MsgErrs,
                     MsgValues as _MsgValues)
from .taskresultset import TaskResultSet
from .taskresult import TaskResult, TaskSeed

MsgFields = _MsgFields()
MsgValues = _MsgValues()
MsgErrs = _MsgErrs()
TaskResultSet = TaskResultSet
TaskResult = TaskResult
TaskSeed = TaskSeed



# -*- coding: utf-8 -*-

import colander
from .fields import MsgFields as F


class _MetaSchema(colander.Schema):
    start_time = colander.SchemaNode(colander.String(), name=F.META_START_TIME)
    end_time = colander.SchemaNode(colander.String(), name=F.META_END_TIME)
    err = colander.SchemaNode(colander.String(), name=F.META_ERR, missing=None)
    msg = colander.SchemaNode(colander.String(), name=F.META_MSG, missing=None)


class BaseTaskSchema(colander.Schema):
    meta = _MetaSchema(name=F.META)


class TestSchema(BaseTaskSchema):
    # A stable schema for testing purposes.
    a = colander.SchemaNode(colander.String(), name="a")
    b = colander.SchemaNode(colander.String(), name="b", missing=None)
    uni = colander.SchemaNode(colander.String(), name="テスト", missing=None)


# ###### Actual Task schema definitions ########

class DomainSchema(BaseTaskSchema):
    domain = colander.SchemaNode(colander.String(), name=F.DOMAIN)


class UrlSchema(BaseTaskSchema):
    url = colander.SchemaNode(colander.String(), name=F.URL)


class HostnameSchema(BaseTaskSchema):
    hostname = colander.SchemaNode(colander.String(), name=F.HOSTNAME)


class WhoisSchema(BaseTaskSchema):
    whois_raw = colander.SchemaNode(colander.String(),
                                    name=F.WHOIS_RAW,
                                    missing=None)


class HttpstatusSchema(BaseTaskSchema):
    http_status = colander.SchemaNode(colander.Integer(),
                                      name=F.HTTP_STATUS,
                                      missing=None)


class IpSchema(BaseTaskSchema):
    ip = colander.SchemaNode(colander.String(), name=F.IP)


class IpSetSchema(colander.Mapping):
    # we allow arbitrary keys here
    pass


class GeoipSchema(BaseTaskSchema):
    cc = colander.SchemaNode(colander.String(),
                             name=F.COUNTRY_CODE,
                             missing=None)
    lat = colander.SchemaNode(colander.Float(),
                              name=F.LAT,
                              missing=None)
    long = colander.SchemaNode(colander.Float(),
                               name=F.LONG,
                               missing=None)



# -*- coding: utf-8 -*-

from .fields import MsgFields as F

# jsonschema v4 doesn't support formats on object keys, so we're stuck with
# regexes to validate. When jsonschema v5 comes out, can look at that.
# This regex as per:
# https://stackoverflow.com/questions/53497/regular-expression-that-matches-va
# lid-ipv6-addresses
ipv4re = "^(\d{1,3}\.){1,3}(\d{1,3})$"
ipv6re = ("^(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:)"
          "{1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}"
          ":){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA"
          "-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0"
          "-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:"
          "[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a"
          "-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25"
          "[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]).){3,3}(25[0-5]|(2[0-4]|1{0,1}"
          "[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}"
          "[0-9]){0,1}[0-9]).){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))$")
ipv4v6re = "{0}|{1}".format(ipv4re, ipv6re)


def build_schema(schema, add_meta=True, type_="object",
                 additionalProperties=False):
    """
    Adds a few necessary embelishments to any schema
    """

    schema["$schema"] = "http://json-schema.org/draft-04/schema#"
    schema["type"] = type_
    schema["additionalProperties"] = additionalProperties
    if add_meta:
        schema["properties"][F.META] = _MetaSchema
    return schema

_MetaSchema = build_schema({
    "properties": {
        F.META_START_TIME: {"type": "string"},
        F.META_END_TIME: {"type": "string"},
        F.META_AGENT_NAME: {"type": ["string", "null"]},
        F.META_TASK_NAME: {"type": "string"},
        F.META_TASK_VERSION: {"type": "string"},
        F.META_ERR: {"type": ["string", "null"]},
        F.META_MSG: {"type": ["string", "null"]},
    }
}, add_meta=False)

# TaskResultSet schema, currently not checked, but defined for completeness
TrsSchema = build_schema({
    "properties": {
        F.TRS_META_JOB_ID: {"type": "number"},
        F.TRS_META_TARGET_ID: {"type": "number"},
    }
}, add_meta=False)

TestSchema = build_schema({
    "properties": {
        "a": {"type": "string"},
        "b": {"type": ["string", "null"]},
        u"レスト": {"type": ["string", "null"]},
    },
    "required": ["a"]
})

# ############ Add schemas from here ################

ArtifactSchema = build_schema({
    "properties": {
        F.PATH: {"type": "string"},
    }
})

AsnipSchema = build_schema({
    "properties": {
        F.ASNIP: {
            "type": "object",
            "patternProperties": {
                ipv4v6re: {
                    "type": "object",
                    "properties": {
                        F.ABUSE_EMAIL: {"type": ["string", "null"]},
                        F.ASN: {"type": ["number", "null"]},
                        F.AS_CC: {"type": ["string", "null"]},
                        F.AS_CIDR: {"type": ["string", "null"]},
                        F.AS_NAME: {"type": ["string", "null"]},
                        F.AS_UPDATE_DATE: {"type": ["string", "null"]},
                        F.NET_CC: {"type": ["string", "null"]},
                        F.NET_CIDR: {"type": ["string", "null"]},
                        F.NET_DESC: {"type": ["string", "null"]},
                        F.NET_NAME: {"type": ["string", "null"]},
                        F.NET_UPDATE_DATE: {"type": ["string", "null"]},
                        F.PHYSICAL_ADDRESS: {"type": ["string", "null"]},
                    },
                }
            },
            "additionalProperties": False,
        },
    }
})

DomainSchema = build_schema({
    "properties": {
        F.DOMAIN: {"type": "string"},
    }
})

# Special schema:
# a bucket for any kinds of misc settings a Job might need: user-agent strings,
# previous file hashes, previous IP addresses
JobsettingsSchema = build_schema({
    "properties": {
    },
}, add_meta=False, additionalProperties=True)

GeoipSchema = build_schema({
    "properties": {
        F.GEOIP: {
            "type": "object",
            "patternProperties": {
                ipv4v6re: {
                    "type": "object",
                    "properties": {
                        F.COUNTRY_CODE: {"type": ["string", "null"]},
                        F.LAT: {"type": ["number", "null"]},
                        F.LONG: {"type": ["number", "null"]},
                    }
                }
            },
            "additionalProperties": False,
        },
    }
})

HostnameSchema = build_schema({
    "properties": {
        F.HOSTNAME: {"type": "string"},
    }
})

HttpstatusSchema = build_schema({
    "properties": {
        F.HTTP_STATUS: {"type": ["number", "null"]},
        F.HTTP_REDIR: {"type": ["string", "null"]},
        F.HTTP_HEADERS: {"type": ["object", "null"]},
        F.IP: {"type": ["string", "null"]},
    }
})

UrlcontentSchema = build_schema({
    "properties": {
        F.HTTP_STATUS: {"type": ["number", "null"]},
        F.FILE_PACK: {"type": ["string", "null"]},
        F.IP: {"type": ["string", "null"]},
    }
})

UrlspidercontentSchema = build_schema({
    "properties": {
        F.HTTP_STATUS: {"type": ["number", "null"]},
        F.FILE_PACK: {"type": ["string", "null"]},
        F.SPIDER_TIMEOUT: {"type": "boolean"},
        F.EXTERNAL_URL_SET: {
            "type": "array",
            "items": {
                "type": ["string", "null"],
            }
        },
        # F.IP: {"type": ["string", "null"]},
    }
})

UrlscreenshotSchema = build_schema({
    "properties": {
        F.FILE_PACK: {"type": ["string", "null"]},
    }
})

IpSingleSchema = build_schema({
    "properties": {
        F.IP: {
            "type": ["string", "null"],
        }
    }
}, add_meta=False)

IpsetSchema = build_schema({
    "properties": {
        F.IPSET: {
            "type": "array",
            "items": {
                "type": ["string", "null"],
                "format": "ipv4",
            }
        }
    }
})

OsfingerprintsSchema = build_schema({
    "properties": {
        F.OS_FINGERPRINTS: {
            "type": "object",
            "patternProperties": {
                ipv4v6re: {
                    "type": ["object", "null"],
                    "properties": {
                        F.PORTS: {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "properties": {
                                    F.PORT_NUM: {"type": ["number"]},
                                    F.PORT_PROTO: {"type": ["string"]},
                                    F.PORT_STATUS: {"type": ["string"]},
                                },
                            },
                        },
                        F.OS_GUESS: {
                            "type": "object",
                            "properties": {
                                F.OS_NAME: {"type": ["string"]},
                                F.OS_GUESS_ACCURACY: {"type": ["number"]},
                                F.OS_FAMILY: {"type": ["string"]},
                                F.OS_GEN: {"type": ["string"]},
                            },
                        },
                        F.STATUS: {
                            "type": "string",
                        },
                    },
                },
            },
            "additionalProperties": False,
        },
    }
})

UrlSchema = build_schema({
    "properties": {
        F.URL: {"type": "string"},
    }
})

VirustotalSchema = build_schema({
    "properties": {
        F.VIRUSTOTAL_RESULT: {"type": ["object", "null"]},
        F.HTTP_STATUS: {"type": ["number", "null"]},
    }
})

WebfingerprintsSchema = build_schema({
    "properties": {
        F.WEB_FINGERPRINTS: {
            "type": ["object", "null"],
            "patternProperties": {
                "[0-9 -_a-zA-Z\:]+": {
                    "properties": {
                        F.CATEGORIES: {
                            "type": ["array"],
                            "items": {
                                "type": ["string", "null"],
                            }
                        }
                    }
                }
            }
        },
        F.IP: {
            "type": ["string", "null"],
        }
    }
})

WhoisSchema = build_schema({
    "properties": {
        F.WHOIS_RAW: {"type": ["string", "null"]},
    }
})



from distutils.core import setup

setup(
    name='dsmsmessages',
    version='0.1.49',
    author='Chris Horsley',
    author_email='chris.horsley@csirtfoundry.com',
    packages=['dsmsmessages', 'dsmsmessages.test'],
    scripts=[],
    url='http://bitbucket.org/dsms/dsmsmessages/',
    license='LICENSE.txt',
    description='Messaging services for DSMS',
    long_description=open('README.txt').read(),
    install_requires=[
        "jsonschema==2.3.0",
        "iso8601==0.1.10",
    ]
)
